/*
 * Copyright (C) 2011 Saulo Popov Zambiasi
 *
 * This source code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Subverse is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: saulopz@gmail.com
 */

#include "Globals.h"
#include "Enjin.h"

#define STOP    -1
#define LEFT    0
#define RIGHT   1
#define UP      2
#define DOWN    3
#define SPEED   10

class Game {
private:
	Enjin *enjin;
	Image *background;
	Image *sphere;
	bool exitGame;
	int x;
	int y;
public:
	Game(); // initialization
	void render();
	void simulateGameWorld();
	void processInput();
	void gameLoop();
	~Game(); // shut down
};

Game::Game() {
	enjin = new Enjin();
	enjin->setFullScreen(false);
	background = enjin->loadImage("../../midia/img/greenBackground.jpg");
	sphere = enjin->loadImage("../../midia/img/sphere.png");
	exitGame = false;
	x = 0;
	y = 0;
	gameLoop();
}

void Game::render() {
	enjin->drawImage(background, enjin->screen, 0, 0, 255);
	enjin->drawImage(sphere, enjin->screen, x, y, 255);
    enjin->drawScreen();
}

void Game::simulateGameWorld(){
    if (enjin->isKeyHeld(SDLK_LEFT)) {
        if (x >= 0) {
            x -= SPEED;
        }
    } else if (enjin->isKeyHeld(SDLK_RIGHT)) {
        if (x < enjin->screen->w - sphere->w) {
            x += SPEED;
        }
    } else if (enjin->isKeyHeld(SDLK_UP)) {
        if (y >= 0) {
            y -= SPEED;
        }
    } else if (enjin->isKeyHeld(SDLK_DOWN)) {
        if (y < enjin->screen->h - sphere->h) {
            y += SPEED;
        }
    } else if (enjin->isKeyDown(SDLK_ESCAPE)) {
        exitGame = true;
    }
}

void Game::processInput() {
    enjin->updateEvents();
}

void Game::gameLoop() {
	while (!exitGame) {
	    render();
		processInput();
        simulateGameWorld();
	}
}

Game::~Game() {
	delete enjin;
}

int main (int argc, char* argv[]){
	Game game;
	return 0;
}
