/*
 * Copyright (C) 2011 Saulo Popov Zambiasi
 *
 * This source code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Subverse is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: saulopz@gmail.com
 */

#include "Globals.h"
#include "Enjin.h"

#define DOWN	0
#define RIGHT	1
#define UP		2
#define LEFT	3

class Game {
    Enjin *enjin;
    Image *background;
    Image *avatar[4][2];
    Music *music;
    Sound *sound[2];
    Font *fontVerdana;
    Font *fontTimes;
    bool exitGame;
    int x, y;
    uint step;
    uint direction;
private:
public:
    Game();
	void render();
	void simulateGameWorld();
	void processInput();
	void gameLoop();
    ~Game();
};

Game::Game() {
    enjin = new Enjin();
    enjin->set("Exemplo: utilizacao de recursos", 800, 600);

    avatar[0][0] = enjin->loadImage("../../midia/img/avatar.0.0.png");
    avatar[0][1] = enjin->loadImage("../../midia/img/avatar.0.1.png");
    avatar[1][0] = enjin->loadImage("../../midia/img/avatar.1.0.png");
    avatar[1][1] = enjin->loadImage("../../midia/img/avatar.1.1.png");
    avatar[2][0] = enjin->loadImage("../../midia/img/avatar.2.0.png");
    avatar[2][1] = enjin->loadImage("../../midia/img/avatar.2.1.png");
    avatar[3][0] = enjin->loadImage("../../midia/img/avatar.3.0.png");
    avatar[3][1] = enjin->loadImage("../../midia/img/avatar.3.1.png");
    background = enjin->loadImage("../../midia/img/greenBackground.jpg");

    music = enjin->loadMusic("../../midia/audio/fernando_strezelecki.mp3");
    enjin->playMusic(music, 1);

    sound[0] = enjin->loadSound("../../midia/audio/Basso.wav");
    sound[1] = enjin->loadSound("../../midia/audio/Glass.wav");

    fontVerdana = enjin->loadFont("../../midia/fonte/verdana.ttf", 12);
    fontTimes = enjin->loadFont("../../midia/fonte/times.ttf", 30);

    exitGame = false;
    x = 0;
    y = 0;
    step = 0;
    direction = 0;
    gameLoop();
}

void Game::render() {
    enjin->drawImage(background, enjin->screen, 0, 0, 255);
    enjin->drawText("Exemplo 04", enjin->screen, fontVerdana, 100, 100, NULL, NULL);
    SDL_Color fg = enjin->color(255, 0, 0);
    SDL_Color bg = enjin->color(0, 255, 0);
    enjin->drawText("Exemplo", enjin->screen, fontTimes, 100, 200, &fg, &bg);
    enjin->drawImage(avatar[direction][step], enjin->screen, x, y, 100);
    enjin->drawScreen();
}

void Game::simulateGameWorld() {
    bool moved = false;
    if (enjin->isKeyDown(SDLK_ESCAPE)) {
        exitGame = true;
    } else if (enjin->isKeyHeld(SDLK_UP)) {
        y -= 10;
        direction = UP;
        moved = true;
    } else if (enjin->isKeyHeld(SDLK_DOWN)) {
        y += 10;
        direction = DOWN;
        moved = true;
    } else if (enjin->isKeyHeld(SDLK_LEFT)) {
        x -= 10;
        direction = LEFT;
        moved = true;
    } else if (enjin->isKeyHeld(SDLK_RIGHT)) {
        x += 10;
        direction = RIGHT;
        moved = true;
    }
    if (enjin->isKeyDown(SDLK_a)) {
        enjin->playSound(sound[0], 0);
    } else if (enjin->isKeyDown(SDLK_s)) {
        enjin->playSound(sound[1], 1);
    }
    if (moved) {
        step = (step == 0) ? 1 : 0;
    }
    enjin->delay(30);
}

void Game::processInput() {
    enjin->updateEvents();
}

void Game::gameLoop() {
    while (!exitGame) {
        render();
        processInput();
        simulateGameWorld();
    }
}

Game::~Game() {
    delete enjin;
}

int main(int argc, char* argv[]) {
    Game game;
    return 0;
}
