/*
 * Copyright (C) 2011 Saulo Popov Zambiasi
 *
 * This source code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Subverse is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: saulopz@gmail.com
 */

#include "Globals.h"
#include "Enjin.h"

#define RIGHT	0
#define LEFT	1
#define SHOT    2
#define UP		3
#define DOWN	4

class Game {
private:
	Enjin *enjin;
	Image *sky;
	Image *city;
	Image *road;
	Image *lamppost;
	Image *ilumination;
	Image *spy[2][3];
	Music *music;
	Sound *soundStep[2];
	Sound *shot;
	int x_skyA;
	int x_skyB;
	int x_cityA;
	int x_cityB;
	int x_roadA;
	int x_roadB;
	int x_lamppost;
	int y;
	int step;
	bool action;
	bool exitGame;
public:
	Game();
	void render();
	void simulateGameWorld();
	void processInput();
	void gameLoop();

    void drawBackGround(Image *image, int &x, int y);
	void runAction(int a);
	~Game();
};

Game::Game() {
	enjin = new Enjin();
	enjin->set("spy", 800, 600);

	spy[0][0] = enjin->loadImage("../../midia/img/spy_0_0.png");
	spy[0][1] = enjin->loadImage("../../midia/img/spy_0_1.png");
	spy[0][2] = enjin->loadImage("../../midia/img/spy_0_2.png");
	spy[1][0] = enjin->loadImage("../../midia/img/spy_1_0.png");
	spy[1][1] = enjin->loadImage("../../midia/img/spy_1_1.png");
	spy[1][2] = enjin->loadImage("../../midia/img/spy_1_2.png");

	sky = enjin->loadImage("../../midia/img/sky.jpg");
	city = enjin->loadImage("../../midia/img/city.png");
	road = enjin->loadImage("../../midia/img/road.png");
	lamppost = enjin->loadImage("../../midia/img/lamppost.png");
	ilumination = enjin->loadImage("../../midia/img/ilumination.png");

	music = enjin->loadMusic("../../midia/audio/fernando_strezelecki.mp3");
	enjin->playMusic(music, 1);

	soundStep[0] = enjin->loadSound("../../midia/audio/step_0.wav");
	soundStep[1] = enjin->loadSound("../../midia/audio/step_1.wav");
	shot = enjin->loadSound("../../midia/audio/shot.wav");

	x_skyA = 0;
	x_skyB = sky->w + 1;
	x_cityA = 0;
	x_cityB = city->w + 1;
	x_roadA = 0;
	x_roadB = road->w;
	x_lamppost = 0;
	y = 400;

    exitGame = false;
	step = 0;
	action = RIGHT;
	gameLoop();
}

void Game::drawBackGround(Image *image, int &x, int y) {
	if (x + image->w < 0) {
		x = x + (image->w * 2);
	}
	if (x > image->w) {
		x = x - (image->w * 2);
	}
	if (x < enjin->screen->w) {
		enjin->drawImage(image, enjin->screen, x, y, 255);
	}
}

void Game::render() {
	if (x_lamppost < -lamppost->w - 300) {
		x_lamppost = enjin->screen->w;
	}
	if (x_lamppost > enjin->screen->w) {
		x_lamppost = -lamppost->w - 300;
	}
	drawBackGround(sky, x_skyA, 0);
	drawBackGround(sky, x_skyB, 0);
	drawBackGround(city, x_cityA, 0);
	drawBackGround(city, x_cityB, 0);
	drawBackGround(road, x_roadA, 350);
	drawBackGround(road, x_roadB, 350);
	enjin->drawImage(ilumination, enjin->screen, x_lamppost, 500, 255);
	enjin->drawImage(spy[action][step], enjin->screen,
		(int) (enjin->screen->w / 2) - (spy[action][step]->w / 2), y, 255);
	enjin->drawImage(lamppost, enjin->screen, x_lamppost, 370, 255);
	enjin->drawScreen();
}

void Game::runAction(int a){
    bool acted = false;
	step = (step > 0) ? 0 : 1;
	switch (a) {
	case RIGHT:
		x_skyA -= 2;
		x_skyB -= 2;
		x_cityA -= 5;
		x_cityB -= 5;
		x_roadA -= 10;
		x_roadB -= 10;
		x_lamppost -= 15;
		action = RIGHT;
		acted = true;
		break;
	case LEFT:
		x_skyA += 2;
		x_skyB += 2;
		x_cityA += 5;
		x_cityB += 5;
		x_roadA += 10;
		x_roadB += 10;
		x_lamppost += 15;
		action = LEFT;
		acted = true;
		break;
	case UP:
		if (y > 340) {
			y -= 10;
			acted = true;
		}
		break;
	case DOWN:
		if (y < 430) {
			y += 10;
			acted = true;
		}
		break;
	case SHOT:
		step = SHOT;
		enjin->playSound(shot, 0);
		acted = true;
		break;
	}
	if ((step != SHOT) && acted) {
		enjin->playSound(soundStep[step], 0);
	}
}

void Game::simulateGameWorld() {
    if (enjin->isKeyDown(SDLK_ESCAPE)) {
        exitGame = true;
    } else if(enjin->isKeyHeld(SDLK_RIGHT) || enjin->isKeyDown(SDLK_RIGHT)) {
        runAction(RIGHT);
    } else if (enjin->isKeyHeld(SDLK_LEFT) || enjin->isKeyDown(SDLK_LEFT)) {
        runAction(LEFT);
    } else if (enjin->isKeyHeld(SDLK_UP) || enjin->isKeyDown(SDLK_UP)) {
        runAction(UP);
    } else if (enjin->isKeyHeld(SDLK_DOWN) || enjin->isKeyDown(SDLK_DOWN)) {
        runAction(DOWN);
    }
    if (enjin->isKeyDown(SDLK_SPACE)) {
        runAction(SHOT);
    }
    enjin->delay(40);
}

void Game::processInput() {
    enjin->updateEvents();
}

void Game::gameLoop() {
    while (!exitGame) {
        render();
        processInput();
        simulateGameWorld();
    }
}

Game::~Game() {
    delete enjin;
}

int main(int argc, char* argv[]) {
	Game game;
	return 0;
}
