/*
 * Copyright (C) 2011 Saulo Popov Zambiasi
 *
 * This source code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Subverse is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: saulopz@gmail.com
 */

#include "Globals.h"
#include "Enjin.h"

#define DOWN	0
#define RIGHT	1
#define UP		2
#define LEFT	3

#define VOID		.
#define BLOCK		0
#define MONSTER		1
#define AVATAR		2

// CLASS ELEMENT **********************************************

class Game;

class Element {
public:
    int type;
    int direction;
    int turn;
    bool walking;
    bool stop;
    Element(int t);
};

Element::Element(int t) {
    type = t;
    turn = 0;
    direction = DOWN;
    stop  = true;
    walking = false;
}

// CLASS GAME ****************************************************

class Game {
private:
	Enjin *enjin;
	Element *gameBoard[16][12];
	Image *sprite[3][4][2];
	Image *ground;
	Sound *sound1;
	Sound *sound2;
	int step;
	int turn;
	bool exitGame;
public:
	Game();
	void build();
	bool moveElement(int x, int y);
	bool runElement(int x, int y);
	void render();
	void gameLoop();
	void simulateGameWorld();
	void processInput();
	~Game();
};

Game::Game() {
	enjin = new Enjin();
	enjin->set("Isometric and Matrix", 800, 600);

	sprite[AVATAR][0][0] = enjin->loadImage("../../midia/img/avatar.0.0.png");
	sprite[AVATAR][0][1] = enjin->loadImage("../../midia/img/avatar.0.1.png");
	sprite[AVATAR][1][0] = enjin->loadImage("../../midia/img/avatar.1.0.png");
	sprite[AVATAR][1][1] = enjin->loadImage("../../midia/img/avatar.1.1.png");
	sprite[AVATAR][2][0] = enjin->loadImage("../../midia/img/avatar.2.0.png");
	sprite[AVATAR][2][1] = enjin->loadImage("../../midia/img/avatar.2.1.png");
	sprite[AVATAR][3][0] = enjin->loadImage("../../midia/img/avatar.3.0.png");
	sprite[AVATAR][3][1] = enjin->loadImage("../../midia/img/avatar.3.1.png");

	sprite[MONSTER][0][0] = enjin->loadImage("../../midia/img/monster.0.0.png");
	sprite[MONSTER][0][1] = enjin->loadImage("../../midia/img/monster.0.1.png");
	sprite[MONSTER][1][0] = enjin->loadImage("../../midia/img/monster.1.0.png");
	sprite[MONSTER][1][1] = enjin->loadImage("../../midia/img/monster.1.1.png");
	sprite[MONSTER][2][0] = enjin->loadImage("../../midia/img/monster.2.0.png");
	sprite[MONSTER][2][1] = enjin->loadImage("../../midia/img/monster.2.1.png");
	sprite[MONSTER][3][0] = enjin->loadImage("../../midia/img/monster.3.0.png");
	sprite[MONSTER][3][1] = enjin->loadImage("../../midia/img/monster.3.1.png");

	for (int i = 0; i < 4; i++) {
		for (int j = 0; j < 2; j++) {
			if ((i == 0) && (j == 0)) {
				sprite[BLOCK][0][0] = enjin->loadImage("../../midia/img/block.0.0.png");
			} else {
				sprite[BLOCK][i][j] = sprite[BLOCK][0][0];
			}
		}
	}
	ground = enjin->loadImage("../../midia/img/grass.jpg");
	sound1 = enjin->loadSound("../../midia/audio/Basso.wav");
	sound2 = enjin->loadSound("../../midia/audio/stepgrass.03.wav");
    step = 0;
    turn = 0;
    exitGame = false;
    build();
    gameLoop();
}

void Game::build() {
    for (int x = 0; x < 16; x++) {
        for (int y = 0; y < 12; y++) {
        	gameBoard[x][y] = NULL;
       	}
	}
    FILE *file;
    if ((file = fopen("ex07.map", "r")) == NULL) {
        cout << "File could not be open." << endl;
        exit(1);
    }
    int x = 0;
    int y = 0;
    while (!feof(file)) {
        bool valid = true;
        int item = fgetc(file);
        switch (item) {
		    case '.': break;
		    case '0': gameBoard[x][y] = new Element(BLOCK); break;
		    case '1': gameBoard[x][y] = new Element(MONSTER); break;
		    case '2': gameBoard[x][y] = new Element(AVATAR); break;
		    default: valid = false;
        }
        if (valid) {
            x++;
            if (x >= 16) {
                x = 0;
                y++;
            }
        }
    }
    fclose(file);
}

bool Game::moveElement(int x, int y) {
    if (gameBoard[x][y] == NULL) return false;
    if (gameBoard[x][y]->stop) return false;

    int xaux = x;
    int yaux = y;
    switch (gameBoard[x][y]->direction) {
		case UP: yaux--; break;
		case DOWN: yaux++; break;
		case LEFT: xaux--; break;
		case RIGHT: xaux++; break;
    }
    if ((xaux < 0) || (xaux >= 16) ||
        (yaux < 0) || (yaux >= 12)) {
        return false;
    }
    Element *eu = gameBoard[x][y];
    Element *ele = gameBoard[xaux][yaux];
    if (eu->type == AVATAR) {
        if (ele != NULL) {
             if (ele->type != MONSTER) {
                return false;
            }
            enjin->playSound(sound1, 0);
        }
    } else if ((eu->type == MONSTER) || (eu->type == BLOCK)) {
        if (ele != NULL) {
            if (ele->type == BLOCK) {
                ele->direction = eu->direction;
                ele->stop = false;
                moveElement(xaux, yaux);
                enjin->playSound(sound2, 0);
            }
            return false;
        }
    } else {
        if (gameBoard[xaux][yaux] != NULL) return false;
    }

    gameBoard[x][y]->walking = true;
    gameBoard[xaux][yaux] = gameBoard[x][y];
    gameBoard[x][y] = NULL;
    return true;
}

bool Game::runElement(int x, int y) {
    if (gameBoard[x][y] == NULL) return false;
    if (gameBoard[x][y]->turn == turn) return false;
    if (step == 1) {
        gameBoard[x][y]->walking = false;
        return false;
    }
    gameBoard[x][y]->turn = turn;
    int direction = gameBoard[x][y]->direction;
    bool stop = true;
    switch (gameBoard[x][y]->type) {
		case AVATAR:
		    if (enjin->isKeyDown(SDLK_UP) || enjin->isKeyHeld(SDLK_UP)) {
		    	direction = UP;
		    	stop = false;
		    } else if (enjin->isKeyDown(SDLK_DOWN) || enjin->isKeyHeld(SDLK_DOWN)) {
		    	direction = DOWN;
		    	stop = false;
		    } else if (enjin->isKeyDown(SDLK_LEFT) || enjin->isKeyHeld(SDLK_LEFT)) {
		    	direction = LEFT;
		    	stop = false;
		    } else if (enjin->isKeyDown(SDLK_RIGHT) || enjin->isKeyHeld(SDLK_RIGHT)) {
		    	direction = RIGHT;
		    	stop = false;
		    }
		    break;
		case MONSTER:
		    direction = (rand() % 4);
		    stop = false;
		    break;
		default:
		    return false;
    }
    gameBoard[x][y]->stop = stop;
    gameBoard[x][y]->direction = direction;
    moveElement(x, y);
    return true;
}

void Game::render() {
	enjin->drawImage(ground, enjin->screen, 0, 0, 255);

    for (int x = 0; x < 16; x++) {
        for (int y = 0; y < 12; y++) {
        	Element *e = gameBoard[x][y];
            if (e != NULL) {
                int posx = (x * 50) - 25;
                int posy = (y * 50) - 25;
                if (e->walking) {
                    switch (e->direction) {
                    case UP   : posy += 25; break;
                    case DOWN : posy -= 25; break;
                    case LEFT : posx += 25; break;
                    case RIGHT: posx -= 25; break;
                    }
                }
                Image *img = sprite[e->type][e->direction][(int) e->walking];
                enjin->drawImage(img, enjin->screen, posx, posy, 255);
            }
        }
    }
	enjin->drawScreen();
}

void Game::simulateGameWorld() {
    for (int x = 0; x < 16; x++) {
        for (int y = 0; y < 12; y++) {
            runElement(x, y);
        }
    }
    step = (step == 0) ? (1) : (0);
    turn++;
    if (enjin->isKeyDown(SDLK_ESCAPE)) {
        exitGame = true;
    }
    enjin->delay(50);
}

void Game::processInput() {
    enjin->updateEvents();
}

void Game::gameLoop() {
    while (!exitGame) {
        render();
        processInput();
        simulateGameWorld();
    }
}

Game::~Game() {
    for (int x = 0; x < 16; x++) {
        for (int y = 0; y < 12; y++) {
            if (gameBoard[x][y] != NULL)
                delete gameBoard[x][y];
        }
    }
    delete enjin;
}

int main(int argc, char* argv[]) {
	Game game;
	return 0;
}
