/*
 * Copyright (C) 2011 Saulo Popov Zambiasi
 *
 * This source code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Subverse is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: saulopz@gmail.com
 */

#include "Globals.h"
#include "Singleton.h"
#include "Enjin.h"

#define DOWN	0
#define LEFT	1
#define UP		2
#define RIGHT	3

#define AVATAR	0
#define	NPC		1

class Element;

// DECLARACAO DA CLASSE GAME ******************************************

class Game {
private:
    Enjin *enjin;
    Image *bigmap;
    Music *music;
    Element *avatar;
    bool exitGame;
public:
    Image *bigmask;
    vector <Element*> elements;
    int x, y;
    uint speed;
    Sound *stepSound[4];

    Game();
    void render();
    void simulateGameWorld();
    void processInput();
    void gameLoop();
    ~Game();
};

// DECLARACAO DA CLASSE PERSONAGEM *************************************

class Element {
private:
    Enjin *enjin;
    Game *game;
    Image *sprites;
    int turn;
    int direction;
    int speed;
    int counter;
public:
    int x, y;
    int h, w;
    int type;

    Element(Game *gm, string name, int type, int v, int posX, int posY);
    void draw();
    bool inRegion(int x, int y, int rX1, int rY1, int rX2, int rY2);
    bool verifyPath(int x, int y);
    bool colide();
    bool move(int dir);
    void run();
};

// IMPLEMENTACAO DA CLASSE PERSONAGEM *************************************

Element::Element(Game *gm, string name, int t, int v, int posX, int posY) {
    enjin = Singleton<Enjin>::getInstance();
    game = gm;
    sprites = enjin->getSprite(name);
    type = t;
    direction = DOWN;
    speed = v;
    turn = 0;
    x = posX;
    y = posY;
    counter = 0;
    w = (int) sprites->w / 7;
    h = (int) sprites->h / 4;
    turn = 0;
}

bool Element::verifyPath(int x, int y) {
    SDL_Color color = enjin->getPixel(game->bigmask, x, y);
    if (enjin->isColor(color, 0, 0, 0)) {
        return false;
    }
    return true;
}

bool Element::inRegion(int x, int y, int rX1, int rY1, int rX2, int rY2) {
    if ((x >= rX1) && (x <= rX2) &&	(y >= rY1) && (y <= rY2)) {
        return true;
    }
    return false;
}

bool Element::colide() {
    bool colision = false;
    for (uint i = 0; i < game->elements.size(); i++) {
        if (game->elements[i] != this) {
            Element *p = game->elements[i];
            switch (direction) {
            case DOWN:
                if (inRegion(x, y+h+speed,
                             p->x, p->y,
                             p->x+w, p->y+h) ||
                        inRegion(x+w, y+h+speed,
                                 p->x, p->y,
                                 p->x+w, p->y+h)) {
                    colision = true;
                }
                break;
            case LEFT:
                if (inRegion(x-speed, y,
                             p->x, p->y,
                             p->x+w, p->y+h) ||
                        inRegion(x-speed, y+h,
                                 p->x, p->y,
                                 p->x+w, p->y+h)) {
                    colision = true;
                }
                break;
            case UP:
                if (inRegion(x, y-speed,
                             p->x, p->y,
                             p->x+w, p->y+h) ||
                        inRegion(x+w, y-speed,
                                 p->x, p->y,
                                 p->x+w, p->y+h)) {
                    colision = true;
                }
                break;
            case RIGHT:
                if (inRegion(x+w+speed, y,
                             p->x, p->y,
                             p->x+w, p->y+h) ||
                        inRegion(x+w+speed, y+h,
                                 p->x, p->y,
                                 p->x+w, p->y+h)) {
                    colision = true;
                }
                break;
            }
        }
    }
    return colision;
}

bool Element::move(int dir) {
    if ((dir < 0) && (dir > RIGHT)) return false;
    direction = dir;
    bool moved = false;
    int x1 = x;
    int y1 = y + (int) (h * 2 / 3);
    int x2 = x + w;
    int y2 = y + h;
    int xaux = x;
    int yaux = y;
    switch (direction) {
    case DOWN:
        if (verifyPath(x1, y2 + speed) &&
                verifyPath(x2, y2 + speed)) {
            yaux += speed;
            moved = true;
        }
        break;
    case LEFT:
        if (verifyPath(x1 - speed, y1) &&
                verifyPath(x1 - speed, y2)) {
            xaux -= speed;
            moved = true;
        }
        break;
    case UP:
        if (verifyPath(x1, y1 - speed) &&
                verifyPath(x2, y1 - speed)) {
            yaux -= speed;
            moved = true;
        }
        break;
    case RIGHT:
        if (verifyPath(x2 + speed, y1) &&
                verifyPath(x2 + speed, y2)) {
            xaux += speed;
            moved = true;
        }
        break;
    }
    if (colide()) {
        moved = false;
    }
    if (moved) {
        x = xaux;
        y = yaux;
        turn++;
        if (turn > 6) {
            turn = 1;
        }
        if ((type == AVATAR) && ((turn == 1) || (turn == 4))) {
            enjin->playSound(game->stepSound[(rand() % 4)], 0);
        }
    } else turn = 0;
    return moved;
}

void Element::draw() {
    int xAux = x;
    int yAux = y;
    if (type == AVATAR) {
        xAux = (int) (enjin->screen->w / 2) - (w / 2);
        yAux = (int) (enjin->screen->h / 2) - (h / 2);
    } else {
        xAux = game->x + x;
        yAux = game->y + y;
    }
    enjin->blitImage(sprites, w*turn, h*direction, w, h,
                     enjin->screen, xAux, yAux, 255);
}

void Element::run() {
    counter++;
    switch (type) {
    case AVATAR :
        if(enjin->isKeyHeld(SDLK_LEFT) || enjin->isJoystickLeft(0)) move(LEFT);
        else if (enjin->isKeyHeld(SDLK_RIGHT) || enjin->isJoystickRight(0)) move(RIGHT);
        else if (enjin->isKeyHeld(SDLK_UP) || enjin->isJoystickUp(0)) move(UP);
        else if (enjin->isKeyHeld(SDLK_DOWN) || enjin->isJoystickDown(0)) move(DOWN);
        else turn = 0;
        break;
    case NPC :
        if ((counter == 20) || (!move(direction))) {
            direction = (rand() % 4);
        }
        break;
    }
    if (counter > 20) counter = 0;
}

// IMPLEMENTACAO DA CLASSE GAME ******************************************

Game::Game() {
    enjin = Singleton<Enjin>::getInstance();
    enjin->set("Movimentacao por cenario", 800, 600);
    bigmap = enjin->loadImage("../../midia/img/bigmap.png");
    bigmask = enjin->loadImage("../../midia/img/bigmask.png");
    enjin->loadSprite("../../midia/img/claudius.png", "claudius");
    enjin->loadSprite("../../midia/img/vanica.png", "vanica");
    enjin->loadSprite("../../midia/img/woman.png", "woman");

    avatar = new Element(this, "claudius", AVATAR, 4, 1100, 1500);
    elements.push_back(avatar);
    elements.push_back(
        new Element(this, "vanica", NPC, 4, 1150, 1500));
    elements.push_back(
        new Element(this, "woman", NPC, 4, 430, 1000));
    x = (int) (enjin->screen->w / 2) - (avatar->x + (avatar->w / 2));
    y = (int) (enjin->screen->h / 2) - (avatar->y + (avatar->h / 2));

    music =
        enjin->loadMusic("../../midia/audio/a_floresta_quadrada_fernando_strezelecki.mp3");
    stepSound[0] = enjin->loadSound("../../midia/audio/stepgrass.01.wav");
    stepSound[1] = enjin->loadSound("../../midia/audio/stepgrass.02.wav");
    stepSound[2] = enjin->loadSound("../../midia/audio/stepgrass.03.wav");
    stepSound[3] = enjin->loadSound("../../midia/audio/stepgrass.04.wav");
    enjin->setMusicVolume(50);
    enjin->playMusic(music, 1);
    exitGame = false;
    gameLoop();
}

void Game::render() {
    x = (int) (enjin->screen->w / 2) - (avatar->x + (avatar->w / 2));
    y = (int) (enjin->screen->h / 2) - (avatar->y + (avatar->h / 2));
    enjin->blitImage(bigmap, x * (-1), y * (-1), enjin->screen->w, enjin->screen->h,
                     enjin->screen, 0, 0, 255);
    for (uint i = 0; i < elements.size(); i++) {
        elements[i]->draw();
    }
    enjin->drawScreen();
}

void Game::processInput() {
    enjin->updateEvents();
}

void Game::simulateGameWorld() {
    for (uint i = 0; i < elements.size(); i++) {
        elements[i]->run();
    }
    enjin->delay(50);
    if (enjin->isKeyDown(SDLK_ESCAPE) || enjin->isJoystickButtonPressed(0, JOYBUTTON_START)) {
        exitGame = true;
    }
}

void Game::gameLoop() {
    while (!exitGame) {
        render();
        processInput();
        simulateGameWorld();
    }
}

Game::~Game() {
    for (uint i = 0; i < elements.size(); i++) {
        delete elements[i];
    }
    enjin->stopMusic();
}

// PROGRAMA PRINCIPAL ****************************************************

int main(int argc, char* argv[]) {
    Game game;
    return 0;
}

