/*
 * Copyright (C) 2011 Saulo Popov Zambiasi
 *
 * This source code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Subverse is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: saulopz@gmail.com
 */

#include "Globals.h"
#include "Enjin.h"

#define RIGHT   0
#define LEFT    1
#define DOWN    2
#define UP      3
#define JUMP    4

#define AVATAR	0
#define DRAGON  1
#define COIN    2
#define CHEST   3

#define GRAVITY 9.999999

string intToString(int d) {
    char *aux = (char*) malloc(255);
    sprintf(aux, "%d", d);
    std::string aux2 = aux;
    return aux2;
}

class Element;

// CLASS GAME DECLARATION ****************************************************

class Game {
private:
    Image *scenario;
    Element *avatar;
    int points;
    Font *font20;
    Font *font60;
    Music *music;
    Color whiteColor;
    bool exitGame;
public:
    Enjin* enjin;
    Image *mask;
    vector <Element*>element;
    Sound *jumpSound;
    Sound *CoinSound;
    Sound *PunchSound;
    Sound *killSound;
    Sound *GameOverSound;
    Sound *victorySound;
    Sound *chestSound;
    bool victory;
    int x;

    Game();
    void render();
    void simulateGameWorld();
    void processInput();
    void gameLoop();
    ~Game();
};

// CLASS ELEMENT DECLARATION ******************************************************

class Element {
private:
    Enjin *enjin;
    Game *game;
    Image *sprites;
    int turn;
    int direction;
    int counter;
    int velocity;
    bool jumping;
    bool inRegion(int x, int y, int rX1, int rY1, int rX2, int rY2);
    void applyGravity();
    float velocityY;
public:
    int x, y;
    int h, w;
    int points;
    int type;
    bool kill;
    bool attacking;
    int attackingCounter;
    bool open;

    Element(Game *gm, string name, int type, int posX, int posY);
    Element* colide(int dir);
    bool freePath(int dir);
    bool pathTest(int x, int y);
    bool move(int dir);
    void jump(int vel);
    void draw();
    void run();
    ~Element();
};

// CLASS ELEMENT IMPLEMENTATION ***********************************************

Element::Element(Game *gm, string name, int t, int posX, int posY) {
    game = gm;
    enjin = game->enjin;
    sprites = enjin->getSprite(name);
    type = t;
    direction = RIGHT;
    turn = 0;
    x = posX;
    y = posY;
    counter = 0;
    velocity = 4;
    velocityY = 0.0;
    jumping = false;
    attacking = false;
    attackingCounter = 0;
    kill = false;
    points = 0;
    w = (int) sprites->w / 6;
    h = (int) sprites->h / 2;
    turn = 0;
    open = false;
}

bool Element::inRegion(int x, int y, int rX1, int rY1, int rX2, int rY2) {
    if ((x >= rX1) && (x <= rX2) &&	(y >= rY1) && (y <= rY2)) {
        return true;
    }
    return false;
}

bool Element::pathTest(int x, int y) {
    if (y <= 0) return true;
    SDL_Color color = enjin->getPixel(game->mask, x, y);
    if (enjin->isColor(color, 0, 0, 0)) {
        return false;
    }
    return true;
}

void Element::applyGravity() {
    // testa se chega ao chao
    if (y > 0) {
        if (!pathTest(x+10, y+h+1) || !pathTest(x+w-10, y+h+1)) {
            velocityY = 0;
            while (!pathTest(x+10, y+h) || !pathTest(x+w-10, y+h)) {
                y--;
            }
            jumping = false;
            return;
        }
    }
    // testa se bate em algo acima
    //if (!pathTest(x+10, y-1) || !pathTest(x+w-10, y-1) || (y < 0)) {
    if (!pathTest(x+10, y-1) || !pathTest(x+w-10, y-1)) {
        velocityY *= (-1);
        while (!pathTest(x+10, y) || !pathTest(x+w-10, y)) {
            y++;
        }
    }
    velocityY = velocityY - (GRAVITY / 10);
    y -= velocityY;
}

Element* Element::colide(int dir) {
    Element *colidesWith = NULL;
    for (uint i = 0; i < game->element.size(); i++) {
        if (game->element[i] != this) {
            Element *e = game->element[i];
            switch (dir) {
            case DOWN:
                if (inRegion(x, y+h+velocity,
                             e->x, e->y,
                             e->x+w, e->y+h) ||
                        inRegion(x+w, y+h+velocity,
                                 e->x, e->y,
                                 e->x+w, e->y+h)) {
                    colidesWith = e;
                }
                break;
            case LEFT:
                if (inRegion(x-velocity, y,
                             e->x, e->y,
                             e->x+w, e->y+h) ||
                        inRegion(x-velocity, y+h,
                                 e->x, e->y,
                                 e->x+w, e->y+h)) {
                    colidesWith = e;
                }
                break;
            case UP:
                if (inRegion(x, y-velocity,
                             e->x, e->y,
                             e->x+w, e->y+h) ||
                        inRegion(x+w, y-velocity,
                                 e->x, e->y,
                                 e->x+w, e->y+h)) {
                    colidesWith = e;
                }
                break;
            case RIGHT:
                if (inRegion(x+w+velocity, y,
                             e->x, e->y,
                             e->x+w, e->y+h) ||
                        inRegion(x+w+velocity, y+h,
                                 e->x, e->y,
                                 e->x+w, e->y+h)) {
                    colidesWith = e;
                }
                break;
            }
        }
    }
    return colidesWith;
}

bool Element::move(int dir) {
    if ((dir < 0) && (dir > LEFT)) return false;
    direction = dir;
    bool moved = false;
    int x1 = x;
    int y1 = y + (int) (h * 2 / 3);
    int x2 = x + w;
    int y2 = y + h;
    int xaux = x;
    switch (direction) {
    case LEFT:
        if (pathTest(x1 - velocity, y1) &&
                pathTest(x1 - velocity, y2)) {
            xaux -= velocity;
            moved = true;
        }
        break;
    case RIGHT:
        if (pathTest(x2 + velocity, y1) &&
                pathTest(x2 + velocity, y2)) {
            xaux += velocity;
            moved = true;
        }
        break;
    }
    Element *e = colide(direction);
    if (e != NULL) {
        moved = false;
        switch (type) {
        case AVATAR:
            if (e->type == CHEST) {
                e->open = true;
            } else if (e->type == COIN) {
                e->kill = true;
                points++;
                enjin->playSound(game->CoinSound, 0);
            } else if (e->type == DRAGON) {
                if (attacking) {
                    e->kill = true;
                } else {
                    kill = true;
                }
            }
            break;
        case DRAGON:
            if (e->type == AVATAR) {
                if (e->attacking) {
                    kill = true;
                } else {
                    e->kill = true;
                }
            }
        case CHEST:
            if (e->type == AVATAR) {
                open = true;
            }
            break;
        }
    }
    if (moved) {
        x = xaux;
        turn++;
        if (turn > 3) {
            turn = 1;
        }
    } else turn = 0;
    return moved;
}

void Element::jump(int vel) {
    if (!jumping) {
        enjin->playSound(game->jumpSound, 0);
        y--;
        velocityY += vel;
        jumping = true;
    }
}

void Element::draw() {
    int xaux = game->x + x;

    int aux = w*turn;
    if (jumping) aux = w*4;
    else if (attacking) aux = w*5;

    if ((xaux < enjin->screen->w) && (xaux+w > 0) && (y < enjin->screen->h) && (y+h > 0)) {
        enjin->blitImage(sprites, aux, h*direction, w, h, enjin->screen, xaux, y, 255);
    }
}

void Element::run() {
    if (game->victory) return;
    if (attackingCounter > 0) {
        attackingCounter--;
    }
    counter++;
    switch (type) {
    case AVATAR :
        if (enjin->isKeyHeld(SDLK_LCTRL) || enjin->isJoystickButtonPressed(0, JOYBUTTON_BALL)) {
            if (!attacking) {
                enjin->playSound(game->PunchSound, 0);
            }
            attacking = true;
            attackingCounter = 10;
        } else if(enjin->isKeyHeld(SDLK_LEFT) || enjin->isJoystickLeft(0)) move(LEFT);
        else if (enjin->isKeyHeld(SDLK_RIGHT) || enjin->isJoystickRight(0)) move(RIGHT);
        else if (enjin->isKeyHeld(SDLK_SPACE) || enjin->isJoystickButtonPressed(0, JOYBUTTON_X)) jump(18);
        else turn = 0;
        if (attacking) {
            if (attackingCounter == 1) {
                attackingCounter = 0;
                attacking = false;
            }
            /*
            if (enjin->isKeyUp(SDLK_LCTRL) || !enjin->isJoystickButtonPressed(0, JOYBUTTON_BALL)) {
                attacking = false;
            }
            */
        }
        break;
    case DRAGON :
        if ((counter == 20) || (!move(direction))) {
            direction = (rand() % 2);
        }
        break;
    case COIN :
        turn++;
        if (turn > 5) turn = 0;
        break;
    case CHEST :
        if (open) {
            if (turn < 5) turn++;
            if (turn == 1) {
                enjin->playSound(game->chestSound, 0);
            }
            if (turn == 5) {
                game->victory = true;
                enjin->playSound(game->victorySound, 0);
            }
        }
        break;
    }
    applyGravity();
    if (counter > 20) counter = 0;
}

Element::~Element() {
}

// CLASS GAME IMPLEMENTATION **********************************************************

Game::Game() {
    enjin = new Enjin();
    enjin->set("Example 09", 800, 600);
    enjin->setFullScreen(false);

    scenario = enjin->loadImage("../../midia/img/scenario.png");
    mask = enjin->loadImage("../../midia/img/scenemask.png");

    enjin->loadSprite("../../midia/img/sonic.png", "avatar");
    enjin->loadSprite("../../midia/img/dragon.png", "dragon");
    enjin->loadSprite("../../midia/img/coin.png", "coin");
    enjin->loadSprite("../../midia/img/chest.png", "chest");
    font20 = enjin->loadFont("../../midia/fonte/times.ttf", 20);
    font60 = enjin->loadFont("../../midia/fonte/times.ttf", 60);
    whiteColor = enjin->color(255, 255, 255);

    avatar = new Element(this, "avatar", AVATAR, 200, 470);
    element.push_back(avatar);

    element.push_back(new Element(this, "coin", COIN, 60, 150));

    element.push_back(new Element(this, "coin", COIN, 180, 45));
    element.push_back(new Element(this, "coin", COIN, 620, 40));
    element.push_back(new Element(this, "coin", COIN, 960, 80));
    element.push_back(new Element(this, "coin", COIN, 940, 100));
    element.push_back(new Element(this, "coin", COIN, 1720, 45));
    element.push_back(new Element(this, "coin", COIN, 1560, 250));
    element.push_back(new Element(this, "coin", COIN, 2100, 465));
    element.push_back(new Element(this, "coin", COIN, 2050, 75));
    element.push_back(new Element(this, "coin", COIN, 2050, 90));
    element.push_back(new Element(this, "coin", COIN, 3190, 250));
    element.push_back(new Element(this, "coin", COIN, 3300, 140));
    element.push_back(new Element(this, "coin", COIN, 3580, 100));
    element.push_back(new Element(this, "coin", COIN, 3500, 470));
    element.push_back(new Element(this, "coin", COIN, 4150, 150));
    element.push_back(new Element(this, "coin", COIN, 4760, 80));
    element.push_back(new Element(this, "coin", COIN, 511, 80));

    element.push_back(new Element(this, "dragon", DRAGON, 570, 530));
    element.push_back(new Element(this, "dragon", DRAGON, 1175, 530));
    element.push_back(new Element(this, "dragon", DRAGON, 1730, 470));
    element.push_back(new Element(this, "dragon", DRAGON, 2900, 530));
    element.push_back(new Element(this, "dragon", DRAGON, 3330, 530));
    element.push_back(new Element(this, "dragon", DRAGON, 3850, 530));
    element.push_back(new Element(this, "dragon", DRAGON, 4400, 470));
    element.push_back(new Element(this, "dragon", DRAGON, 510, 530));

    element.push_back(new Element(this, "chest", CHEST, 5350, 480));

    jumpSound = enjin->loadSound("../../midia/audio/jump.wav");
    CoinSound = enjin->loadSound("../../midia/audio/coin.wav");
    killSound = enjin->loadSound("../../midia/audio/Basso.wav");
    PunchSound = enjin->loadSound("../../midia/audio/punch.wav");
    GameOverSound = enjin->loadSound("../../midia/audio/gameover.wav");
    chestSound = enjin->loadSound("../../midia/audio/chest.wav");
    victorySound = enjin->loadSound("../../midia/audio/victory.wav");

    music = enjin->loadMusic("../../midia/audio/fernando_strezelecki.mp3");
    enjin->setMusicVolume(100);
    enjin->playMusic(music, 9);

    exitGame = false;
    victory = false;
    x = 0;
    points = 0;
    gameLoop();
}

void Game::render() {
    if (avatar != NULL) {
        x = (int) (enjin->screen->w / 2) - (avatar->x + (avatar->w / 2));
        points = avatar->points;
    }
    if (x > 0) {
        x = 0;
    } else if (x + scenario->w < enjin->screen->w) {
        x = enjin->screen->w - scenario->w;
    }
    enjin->blitImage(scenario, x * (-1), 0, enjin->screen->w, scenario->h,
                     enjin->screen, 0, 0, 255);
    for (uint i = 0; i < element.size(); i++) {
        element[i]->draw();
    }

    string strPoints = "Points: ";
    strPoints += intToString(points);
    enjin->drawText(strPoints, enjin->screen, font20, 5, 1, &whiteColor, NULL);
    if (avatar == NULL) {
        enjin->stopMusic();
        enjin->drawText("Game Over", enjin->screen, font60, 300, 250, &whiteColor, NULL);
    }
    if (victory) {
        enjin->stopMusic();
        enjin->drawText("You win!", enjin->screen, font60, 300, 250, &whiteColor, NULL);
    }
    enjin->drawScreen();
}

void Game::simulateGameWorld() {
    for (uint i = 0; i < element.size(); i++) {
        element[i]->run();
    }
    for (uint i = 0; i < element.size(); i++) {
        if (element[i]->kill) {
            if (element[i] == avatar) {
                avatar = NULL;
                enjin->playSound(GameOverSound, 0);
            } else if (element[i]->type != COIN) {
                enjin->playSound(killSound, 0);
            }
            element.erase(element.begin()+i);
        }
    }
    enjin->delay(50);
    if (enjin->isKeyDown(SDLK_ESCAPE) || enjin->isJoystickButtonPressed(0, JOYBUTTON_START)) {
        exitGame = true;
    }
}

void Game::processInput() {
    enjin->updateEvents();
}

void Game::gameLoop() {
    while (!exitGame) {
        render();
        processInput();
        simulateGameWorld();
    }
}

Game::~Game() {
    delete enjin;
}

int main(int argc, char* argv[]) {
    Game g;
    return 0;
}
