/*
 * Copyright (C) 2011 Saulo Popov Zambiasi
 *
 * This source code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Subverse is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: saulopz@gmail.com
 */

#include "Globals.h"
#include "Enjin.h"

string intToString(int d) {
	char *aux = (char*) malloc(255);
	sprintf(aux, "%d", d);
	std::string aux2 = aux;
	return aux2;
}

class Game {
private:
    Enjin* enjin;
    Image *background;
    Image *sphere;
    Font *font20;
    Color whiteColor;
    string joystickEvents;
    bool exitGame;
    int x, y;
public:
    Game();
	void render();
	void simulateGameWorld();
	void processInput();
	void gameLoop();
    ~Game();
};

Game::Game() {
    enjin = new Enjin();
    enjin->set("Joystik", 800, 600);

    background = enjin->loadImage("../../midia/img/greenBackground.jpg");
    sphere = enjin->loadImage("../../midia/img/sphere.png");

    font20 = enjin->loadFont("../../midia/fonte/times.ttf", 20);
    whiteColor = enjin->color(255, 255, 255);
    x = 0;
    y = 0;
    joystickEvents = "";
    exitGame = false;
    gameLoop();
}

void Game::render() {
    if (x < 0) x = 0;
    if (x + sphere->w > enjin->screen->w) x = enjin->screen->w - sphere->w;
    if (y < 0) y = 0;
    if (y + sphere->h > enjin->screen->h) y = enjin->screen->h - sphere->h;

    enjin->drawImage(background, enjin->screen, 0, 0, 255);
    string out = "Eventos Joystick: " + joystickEvents;
    enjin->drawText(out, enjin->screen, font20, 100, 100, &whiteColor, NULL);
    enjin->drawImage(sphere, enjin->screen, x, y, 255);
    enjin->drawScreen();
}

void Game::simulateGameWorld() {
    joystickEvents = "";

    if (enjin->isJoystickUp(0)) y-=5;
    if (enjin->isJoystickDown(0)) y+=5;
    if (enjin->isJoystickLeft(0)) x-=5;
    if (enjin->isJoystickRight(0)) x+=5;

    for (int i = 0; i < JOY_MAX_BUTTONS; i++) {
        bool aux = enjin->isJoystickButtonPressed(0, i);
        if (aux) {
            joystickEvents += intToString(i);
            joystickEvents += " ";
            cout << "Eventos joystick: " << i << endl;
        }
    }
    if (enjin->isKeyDown(SDLK_ESCAPE) || enjin->isJoystickButtonPressed(0, JOYBUTTON_START)) {
        exitGame = true;
    }
}

void Game::processInput() {
    enjin->updateEvents();
}

void Game::gameLoop() {
    while (!exitGame) {
        render();
        processInput();
        simulateGameWorld();
    }
}

Game::~Game() {
    delete enjin;
}

int main(int argc, char* argv[]) {
	Game game;
	return 0;
}
